library ieee;
use ieee.std_logic_1164.all;

entity control is
 port (clk : in std_logic;
       areset : in std_logic;
       start : in std_logic;
       clear : in std_logic;
       ready : out std_logic;
       countup : out std_logic;
       clearout : out std_logic
      );
end entity control;

architecture fsm of control is
type state_type is (rest_st, idle_st, run_st, run1_st,
                    hold_st, hold1_st);
signal state : state_type;
begin
    nslreg: process (clk, areset) is
    begin
        if areset = '1' then
            state <= rest_st;
        elsif rising_edge(clk) then
            case state is
                when rest_st =>  if clear = '0' and start = '0' then
                                     state <= idle_st;
                                 end if;
                when idle_st =>  if clear = '1' then
                                     state <= idle_st;
                                 elsif start = '1' then
                                     state <= run_st;
                                 end if;
                when run_st =>   if start = '0' then
                                     state <= run1_st;
                                 end if;
                when run1_st =>  if start = '1' then
                                     state <= hold_st;
                                 end if;
                when hold_st =>  if start = '0' then
                                     state <= hold1_st;
                                 end if;
                when hold1_st => if clear = '1' then
                                     state <= idle_st;
                                 elsif start = '1' then
                                      state <= run_st;
                                 end if;
                when others => state <= rest_st;
            end case;
        end if;
    end process;
 
    ol: process (state) is
    begin
        clearout <= '0';
        countup <= '0';
        ready <= '0';
        case state is
            when rest_st =>  clearout <= '1';
            when idle_st =>  clearout <= '1';
                             ready <= '1';
            when run_st =>   countup <= '1';
            when run1_st =>  countup <= '1';
            when hold_st =>  null;
            when hold1_st => ready <= '1';
            when others =>   null;
        end case;
    end process;
  
--    ol: process (state) is
--    begin
--        case state is
--            when rest_st =>  clearout <= '1';
--                             countup <= '0';
--                             ready <= '0';
--            when idle_st =>  clearout <= '1';
--                             countup <= '0';
--                             ready <= '1';
--            when run_st =>   clearout <= '0';
--                             countup <= '1';
--                             ready <= '0';
--            when run1_st =>  clearout <= '0';
--                             countup <= '1';
--                             ready <= '0';
--            when hold_st =>  clearout <= '0';
--                             countup <= '0';
--                             ready <= '0';
--            when hold1_st => clearout <= '0';
--                             countup <= '0';
--                             ready <= '1';
--            when others =>   clearout <= '0';
--                             countup <= '0';
--                             ready <= '0';
--        end case;
--    end process;
  
end architecture fsm;