library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity counter is
 generic (sysfreq : integer := 50000000;
          watchfreq : integer := 100
         );
 port (clk : in std_logic;
       areset : in std_logic;
       clear : in std_logic;
       countup : in std_logic;
       tensec : out unsigned(3 downto 0);
       sec : out unsigned(3 downto 0);
       secten : out unsigned(3 downto 0);
       sechund : out unsigned(3 downto 0);
       pulse : out std_logic
      );
end entity counter;

architecture rtl of counter is
signal tensec_int : unsigned(3 downto 0);
signal sec_int : unsigned(3 downto 0);
signal secten_int : unsigned(3 downto 0);
signal sechund_int : unsigned(3 downto 0);
signal prescaler_int : integer range 0 to (sysfreq/watchfreq-1);
signal pulsecount_int : integer range 0 to watchfreq/2-1;
signal pulse_int : std_logic;
begin
    process (clk, areset, prescaler_int) is
    begin
        if areset = '1' then
            tensec_int <= (others => '0');
            sec_int <= (others => '0');
            secten_int <= (others => '0');
            sechund_int <= (others => '0');
            prescaler_int <= 0;
            pulse_int <= '0';
            pulsecount_int <= 0;
        elsif rising_edge(clk) then
            if clear = '1' then
                tensec_int <= (others => '0');
                sec_int <= (others => '0');
                secten_int <= (others => '0');
                sechund_int <= (others => '0');
                prescaler_int <= 0;
                pulse_int <= '1';
                pulsecount_int <= 0;
            elsif countup = '1' then
                if prescaler_int = (sysfreq/watchfreq-1) then
                    if pulsecount_int = (watchfreq/2-1) then
                        pulsecount_int <= 0;
                        pulse_int <= not pulse_int;
                    else
                        pulsecount_int <= pulsecount_int + 1;
                    end if;
                    prescaler_int <= 0;
                    if sechund_int = "1001" then
                        sechund_int <= "0000";
                        if secten_int = "1001" then
                            secten_int <= "0000";
                            if sec_int = "1001" then
                                sec_int <= "0000";
                                if tensec_int = "1001" then
                                    tensec_int <= "0000";
                                else
                                    tensec_int <= tensec_int + 1;
                                end if;
                            else
                                sec_int <= sec_int + 1;
                            end if;
                        else
                            secten_int <= secten_int + 1;
                        end if;
                     else
                         sechund_int <= sechund_int + 1;
                     end if;
                 else
                     prescaler_int <= prescaler_int + 1;
                 end if;
              else
                  pulse_int <= '1';
             end if;
        end if;

    end process;
 
    tensec <= tensec_int;
    sec <= sec_int;
    secten <= secten_int;
    sechund <= sechund_int;
    pulse <= pulse_int;

end architecture rtl;